package com.bastionsdk.ane.android.helpers;

import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.bastionsdk.android.Application;
import com.bastionsdk.android.CodeErrorInfo;
import com.bastionsdk.android.Feature;
import com.bastionsdk.android.Offer;
import com.bastionsdk.android.Resource;

public class JSONMapper
{
	public static JSONObject toJSON(Offer offer) throws JSONException
	{
		if( offer == null )
		{
			throw new NullPointerException("offer==null");
		}
		
		JSONObject json = new JSONObject();
		
		json.put("r", offer.getOfferReference());
		json.put("data", additionalParametersToJSON(offer.getOfferAdditionalParameters()));
		json.put("feat", featuresToJSON(offer.getFeatures()));
		json.put("res", resourcesToJSON(offer.getResources()));
		json.put("bundles", new JSONArray(offer.getBundleReferences()));
		
		return json;
	}
	
	public static JSONObject toJSON(CodeErrorInfo infos) throws JSONException
	{
		if( infos == null )
		{
			throw new NullPointerException("infos==null");
		}
		
		JSONObject json = new JSONObject();
		
		json.put("type", infos.getType().toString());
		
		if( infos.hasMissingApplications() )
		{
			json.put("missingApps", missingAppsToJSON(infos.getMissingApplications()));
		}
		
		return json;
	}
	
// ------------------------------------------>
	
	private static JSONArray missingAppsToJSON(List<Application> missingApplications) throws JSONException
	{
		if( missingApplications == null )
		{
			throw new NullPointerException("missingApplications==null");
		}
		
		JSONArray json = new JSONArray();
		
		for( Application app : missingApplications )
		{
			json.put(missingAppToJSON(app));
		}
		
		return json;
	}
	
	private static JSONObject missingAppToJSON(Application app) throws JSONException
	{
		if( app == null )
		{
			throw new NullPointerException("app==null");
		}
		
		JSONObject json = new JSONObject();
		
		if( app.hasBundleId() )
		{
			json.put("bundleId", app.getBundleId());
		}
		
		if( app.hasScheme() )
		{
			json.put("scheme", app.getScheme());
		}
		
		return json;
	}

	
// ------------------------------------------>

	public static JSONArray resourcesToJSON(List<Resource> resources) throws JSONException
	{
		if( resources == null )
		{
			throw new NullPointerException("resources == null");
		}
		
		JSONArray array = new JSONArray();
		
		for(Resource res : resources)
		{
			array.put(resourceToJSON(res));
		}
		
		return array;
	}

	private static JSONObject resourceToJSON(Resource resource) throws JSONException
	{
		if( resource == null )
		{
			throw new NullPointerException("resource==null");
		}
		
		JSONObject json = new JSONObject();
		
		json.put("r", resource.getReference());
		json.put("q", resource.getQuantity());
		
		if( resource.isInBundle() )
		{
			json.put("b", resource.getBundleReference());
		}
		
		return json;
	}

	public static JSONArray featuresToJSON(List<Feature> features) throws JSONException
	{
		if( features == null )
		{
			throw new NullPointerException("features == null");
		}
		
		JSONArray array = new JSONArray();
		
		for(Feature feature : features)
		{
			array.put(featureToJSON(feature));
		}
		
		return array;
	}

	private static JSONObject featureToJSON(Feature feature) throws JSONException
	{
		if( feature == null )
		{
			throw new NullPointerException("feature == null");
		}
		
		JSONObject json = new JSONObject();
		
		json.put("r", feature.getReference());
		
		if( feature.isInBundle() )
		{
			json.put("b", feature.getBundleReference());
		}
		
		if( feature.hasValue() )
		{
			json.put("val", feature.getValue());
		}
		
		if( !feature.isLifetime() )
		{
			json.put("ttl", feature.getTTL());
		}
				
		return json;
	}

	private static JSONArray additionalParametersToJSON(Map<String, String> parameters) throws JSONException
	{
		if( parameters == null )
		{
			throw new NullPointerException("parameters==null");
		}
		
		JSONArray array = new JSONArray();
		
		for(String n : parameters.keySet())
		{
			JSONObject param = new JSONObject();
			
			param.put("n", n);
			param.put("v", parameters.get(n));
			
			array.put(param);
		}
		
		return array;
	}
}
